@@ -35,6 +35,7 @@ if TYPE_CHECKING:
     from narwhals._polars.expr import PolarsExpr
     from narwhals._polars.namespace import PolarsNamespace
     from narwhals.dtypes import DType
+    from narwhals.series import Series
     from narwhals.typing import Into1DArray
     from narwhals.typing import MultiIndexSelector
     from narwhals.typing import _1DArray
@@ -193,6 +194,9 @@ class PolarsSeries:
         native = pl.Series(data if is_numpy_array_1d(data) else [data])
         return cls.from_native(native, context=context)
 
+    def to_narwhals(self) -> Series[pl.Series]:
+        return self._version.series(self, level="full")
+
     def _with_native(self, series: pl.Series) -> Self:
         return self.__class__(
             series, backend_version=self._backend_version, version=self._version
@@ -516,7 +520,7 @@ class PolarsSeries:
         except Exception as e:  # noqa: BLE001
             raise catch_polars_exception(e, self._backend_version) from None
 
-    def hist(
+    def hist(  # noqa: C901, PLR0912
         self,
         bins: list[float | int] | None,
         *,
